@@ -42,26 +42,21 @@ void CameraUI::ProcessCropAndSend(QImage img, QPoint topLeft, bool correctVignet
     }
 
     QImage croppedImageBW = ConvertToGrayScale(img);
-    crop_filename = "/home/user/MyDocs/photo_cropped.jpg";
+    crop_filename = (QDir::homePath() + QString("/.mir-translator/MIRATS_cropped.jpg")).toAscii().data();
     croppedImageBW.save(crop_filename.c_str(), 0, 80);
 
-    /*
-    // Totally just TESTING HERE for the integral image
-    AdaptiveThresholder thresholder;
-    thresholder.Init(croppedImageBW);
-    thresholder.AdaptivelyThreshold(30, 0.9).save("/home/user/MyDocs/photo_adaptively_thresholded.jpg");
-    // END TESTING
-    */
-
     // Get the Amazon EC2 instance's IP and make the request to EC2
-    QHostInfo ec2host = QHostInfo::fromName("ec2-50-17-193-134.compute-1.amazonaws.com");
+    QHostInfo ec2host = QHostInfo::fromName("mir.mhsueh.com");
     QHostAddress ec2addr = ec2host.addresses().first();
     //ocr_req = new OCRRequest("192.168.2.14", 20248, (char *) crop_filename.c_str());
 
 
     //The OCRRequest is instantiated here. Hardcoded line for local server. Use the above line for EC2 server.
     ocr_req = new OCRRequest(ec2addr.toString(), 20248, (char *) crop_filename.c_str());
-    qDebug() << ec2addr.toString();
+    m_just_timed_out = false;
+    //qDebug() << ec2addr.toString();
+    connect(ocr_req, SIGNAL(TimedOutSignal(OCRRequest *)), this, SLOT(ShowTimedOutDiscDialog(OCRRequest *)));
+    connect(ocr_req, SIGNAL(ErrorConnecting(OCRRequest *)), this, SLOT(ShowErrorConnectingDialog(OCRRequest*)));
 
     // Connect the ocr_req's SIGNAL OCR_Finished to the SLOT for cameraUI to set the OCR results
     connect(ocr_req, SIGNAL(OCR_finished(QString)), this, SLOT(SetOCRText(QString)));
@@ -73,24 +68,64 @@ void CameraUI::ProcessCropAndSend(QImage img, QPoint topLeft, bool correctVignet
     //Set the source language and then send the OCR request to the server using ocr_req
     ocr_req->SetLang(sourceCBox->currentText());
     ocr_req->MakeOCRRequest();
+    connect(quitButton, SIGNAL(clicked()), ocr_req->server_socket, SLOT(disconnectFromHostImplementation()));
+    connect(sourceCBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CropChanged())); // Changing languages shoudl re-enable request button
+    connect(destCBox, SIGNAL(currentIndexChanged(int)), this, SLOT(CropChanged()));
 }
 
 // Populates the language combo box cb
 void CameraUI::PopulateLangCBox(QComboBox* cb){
     cb->clear();
-    QHash<QString, QString>::iterator it;
-    for(it = langStrings.begin(); it != langStrings.end(); it++){
-        cb->addItem(it.key());
+    QList<QString> keys = langStrings.keys();
+    qSort(keys);
+    keys.removeAll(QString("English"));
+    keys.insert(0, QString("English")); // Do this to put English at the front.
+    QList<QString>::iterator it;
+    for(it = keys.begin(); it != keys.end(); it++){
+        cb->addItem(*it);
     }
 }
 
 // Initialize the mappings from the combo box text to Google API strings
 void CameraUI::InitLangStrings(){
-    langStrings["JPN"] = "ja";
-    langStrings["CHI-t"] = "zh-TW";
-    langStrings["CHI-s"] = "zh-CN";
-    langStrings["KOR"] = "ko";
-    langStrings["ENG"] = "en";
+    langStrings["English"] = "en";
+    langStrings["Japanese"] = "ja";
+    langStrings["Chinese (tra)"] = "zh-TW";
+    langStrings["Chinese (sim)"] = "zh-CN";
+    langStrings["Korean"] = "ko";
+
+    langStrings["Arabic"] = "ar";
+    langStrings["Bulgarian"] = "bg";
+    langStrings["Catalan"] = "ca";
+    langStrings["Czech"] = "cs";
+    langStrings["Danish"] = "da";
+    langStrings["German"] = "de";
+    langStrings["Greek"] = "el";
+
+    langStrings["Finnish"] = "fi";
+    langStrings["French"] = "fe";
+    langStrings["Hebrew"] = "iw";
+    langStrings["Hungarian"] = "hu";
+    langStrings["Indonesian"] = "id";
+    langStrings["Italian"] = "it";
+    langStrings["Latvian"] = "lv";
+    langStrings["Lituanian"] = "lt";
+    langStrings["Dutch"] = "nl";
+    langStrings["Norwegian"] = "no";
+    langStrings["Polish"] = "pl";
+    langStrings["Portuguese"] = "pt";
+    langStrings["Romanian"] = "ro";
+    langStrings["Russian"] = "ru";
+    langStrings["Slovakian"] = "sk";
+    langStrings["Slovenian"] = "sl";
+    langStrings["Spanish"] = "es";
+    langStrings["Serbian"] = "sr";
+    langStrings["Swedish"] = "sv";
+    langStrings["Filipino"] = "tl";
+    langStrings["Thai"] = "th";
+    langStrings["Turkish"] = "tr";
+    langStrings["Ukrainian"] = "uk";
+    langStrings["Vietnamese"] = "vi";
 }
 
 // Make a retranslation request
@@ -98,7 +133,7 @@ void CameraUI::Make_Retrans_Request(){
     retransEnabled = false;
     retransButton->setEnabled(false);
     retransButton->updateGeometry();
-    statusLabel->setText("Retranslating");
+    statusLabel->setText("- Translating -");
     ocr_req->SetInputPath(crop_filename.c_str());
     ocr_req->Make_Retrans_Request(resultView->toPlainText(), langStrings[sourceCBox->currentText()], langStrings[destCBox->currentText()]);
 }
@@ -205,7 +240,7 @@ void CameraUI::clearResults() {
 void CameraUI::SetTranslationText(QString text){
     bTranslationComplete = true;
     textButton->setEnabled(true);
-    statusLabel->setText("Translation done");
+    statusLabel->setText("- Done -");
     translatedText = text;
     translateView->setText(text);
 }
@@ -336,7 +371,7 @@ void CameraUI::send() {
     textButton->show();
     textButton->setEnabled(false);
     statusLabel->show();
-    statusLabel->setText("Translation in progress...");
+    statusLabel->setText("- Translating -");
     qApp->processEvents();
 
     // We will need to discard any existing OCRReqs because we will be creating a new one
@@ -375,12 +410,16 @@ void CameraUI::HandleResultCursorChange(){
         if(cursor.position() > oldTextCursor.selectionEnd()){
             cursor.setPosition(oldTextCursor.selectionStart());
             cursor.setPosition(pos, QTextCursor::KeepAnchor);
-            qDebug("old start:%d, old end:%d", oldTextCursor.selectionStart(), oldTextCursor.selectionEnd());
+            //qDebug("old start:%d, old end:%d", oldTextCursor.selectionStart(), oldTextCursor.selectionEnd());
         }else if(cursor.position() < oldTextCursor.selectionStart()){
             cursor.setPosition(oldTextCursor.selectionEnd());
             cursor.setPosition(pos, QTextCursor::KeepAnchor);
         }
         resultView->setTextCursor(cursor);
+        alt_table->setRangeSelected(QTableWidgetSelectionRange(0,0,0,num_alts - 1), false);
+        for(int i = 0; i < num_alts; i++){
+            alt_table->item(0, i)->setText("");
+        }
     }else{
 
         // Set the background of the highlighted text to GREY (different from the default)
@@ -440,7 +479,8 @@ void CameraUI::AdjustAltTableSize(){
     }
 
     alt_table->setFixedWidth(total_width);
-    alt_table->setFixedHeight(alt_table->rowHeight(0));
+    alt_table->setRowHeight(0, 40);
+    alt_table->setFixedHeight(40);
     alt_table->updateGeometry();
 }
 
@@ -451,8 +491,8 @@ void CameraUI::EnableRetranslate(){
 }
 
 void CameraUI::viewText() {
-    quitButton->hide(); // Hide the close button for now to make room for the alt char picture preview
-    statusLabel->hide(); // Hide the status message for now to make room for the alt char picture preview
+    quitButton->show();
+    statusLabel->show();
     bbView->show(); // Show the bb image
     overlay->hide();
     imageView->hide();
@@ -473,7 +513,6 @@ void CameraUI::viewText() {
     destLangLabel->hide();
     destCBox->hide();
     sourceCBox->hide();
-    statusLabel->hide();
     manualCorrectionLabel->show();
 
     HandleResultCursorChange();
@@ -606,7 +645,7 @@ void CameraUI::ViewPhoto() {
     if(OCRdText.isEmpty()){
         statusLabel->setText("Select area to translate");
     }else{
-        statusLabel->setText("Translation done");
+        statusLabel->setText("- Done -");
     }
     retransButton->hide();
     overlay->hide();
@@ -615,7 +654,12 @@ void CameraUI::ViewPhoto() {
     if(translatedText.length() != 0 || OCRdText.length() != 0){
         textButton->show();
         sendButton->hide();
-        statusLabel->setText("Translation done");
+        statusLabel->setText("- Done -");
+    }else if(m_just_timed_out){
+        textButton->hide();
+        sendButton->show();
+        statusLabel->setText("Select area to translate");
+        m_just_timed_out = false;
     }else{
         // We are entering the photo view fresh off a snap from the camera screen
         textButton->hide();
@@ -689,6 +733,7 @@ void CameraUI::StartResegmentDialog(){
         resegmentDialog->deleteLater();
     }
     resegmentDialog = new QDialog();
+    resegmentDialog->setWindowTitle("Manual Correction");
     QHBoxLayout resegmentDialogLayout;
     QVBoxLayout resegmentButtons;
     QHBoxLayout thresholdRadioButtonLayout;
@@ -741,6 +786,36 @@ void CameraUI::StartResegmentDialog(){
     ResetAdaptiveThresholder();
 }
 
+void CameraUI::ShowTimedOutDiscDialog(OCRRequest * req){
+    // Don't need to delete here. Will be deleted on new send request.
+    ocr_req->blockSignals(true);
+    QDialog * timedOutDiscDialog = new QDialog();
+    timedOutDiscDialog->setWindowTitle("Timed Out");
+    timedOutDiscDialog->setModal(true);
+    QLabel l("Request Timed Out. Please check your connection or try translating a smaller region.", timedOutDiscDialog);
+    l.setWordWrap(true);
+    l.show();
+    timedOutDiscDialog->exec();
+    timedOutDiscDialog->deleteLater();
+    m_just_timed_out = true;
+    ViewPhoto();
+}
+
+void CameraUI::ShowErrorConnectingDialog(OCRRequest *req){
+    // Don't need to delete here. Will be deleted on new send request.
+    ocr_req->blockSignals(true);
+    QDialog * errorConnectingDialog = new QDialog();
+    errorConnectingDialog->setWindowTitle("Connection Error");
+    errorConnectingDialog->setModal(true);
+    QLabel l("Error connecting to the server. Please check your connection.", errorConnectingDialog);
+    l.setWordWrap(true);
+    l.show();
+    errorConnectingDialog->exec();
+    errorConnectingDialog->deleteLater();
+    m_just_timed_out = true;
+    ViewPhoto();
+}
+
 BoundingBoxCoords CameraUI::GetSelectionBB(){
     QTextCursor cursor = resultView->textCursor();
     BoundingBoxCoords maxBoundingBox(bbScene->width(), bbScene->height(), 0, 0);
@@ -762,7 +837,7 @@ void CameraUI::SendResegmentReq(){
     //QImage resegmentCropImage(crop_filename.c_str());
     QImage resegmentCropImage = bbScenePixmapItem->pixmap().toImage();
     resegmentCropImage = resegmentCropImage.copy(resegmentBB.toRect());
-    QString reseg_filename = QString("/home/user/MyDocs/cropped_reseg.jpg");
+    QString reseg_filename = QDir::homePath() + QString("/.mir-translator/MIRATS_reseg.jpg");
     resegmentCropImage.save(reseg_filename);
 
     if(resegment_ocr_req != 0){
@@ -770,7 +845,7 @@ void CameraUI::SendResegmentReq(){
     }
 
     // Get the Amazon EC2 instance's IP and make the request to EC2
-    QHostInfo ec2host = QHostInfo::fromName("ec2-50-17-193-134.compute-1.amazonaws.com");
+    QHostInfo ec2host = QHostInfo::fromName("mir.mhsueh.com");
     QHostAddress ec2addr = ec2host.addresses().first();
 
     //resegment_ocr_req = new OCRRequest("192.168.2.14", 20248, reseg_filename);
@@ -784,6 +859,7 @@ void CameraUI::SendResegmentReq(){
     ResegOffsetY = resegmentCropItem->cropRect().top();
 
     resegment_ocr_req->MakeOCRRequest();
+    connect(quitButton, SIGNAL(clicked()), resegment_ocr_req->server_socket, SLOT(disconnectFromHostImplementation()));
 }
 
 void CameraUI::EnableThresholdSlider(){
